#!/bin/sh
set -e
set -u
set -o pipefail

function on_error {
  echo "$(realpath -mq "${0}"):$1: error: Unexpected failure"
}
trap 'on_error $LINENO' ERR

if [ -z ${UNLOCALIZED_RESOURCES_FOLDER_PATH+x} ]; then
  # If UNLOCALIZED_RESOURCES_FOLDER_PATH is not set, then there's nowhere for us to copy
  # resources to, so exit 0 (signalling the script phase was successful).
  exit 0
fi

mkdir -p "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}"

RESOURCES_TO_COPY=${PODS_ROOT}/resources-to-copy-${TARGETNAME}.txt
> "$RESOURCES_TO_COPY"

XCASSET_FILES=()

# This protects against multiple targets copying the same framework dependency at the same time. The solution
# was originally proposed here: https://lists.samba.org/archive/rsync/2008-February/020158.html
RSYNC_PROTECT_TMP_FILES=(--filter "P .*.??????")

case "${TARGETED_DEVICE_FAMILY:-}" in
  1,2)
    TARGET_DEVICE_ARGS="--target-device ipad --target-device iphone"
    ;;
  1)
    TARGET_DEVICE_ARGS="--target-device iphone"
    ;;
  2)
    TARGET_DEVICE_ARGS="--target-device ipad"
    ;;
  3)
    TARGET_DEVICE_ARGS="--target-device tv"
    ;;
  4)
    TARGET_DEVICE_ARGS="--target-device watch"
    ;;
  *)
    TARGET_DEVICE_ARGS="--target-device mac"
    ;;
esac

install_resource()
{
  if [[ "$1" = /* ]] ; then
    RESOURCE_PATH="$1"
  else
    RESOURCE_PATH="${PODS_ROOT}/$1"
  fi
  if [[ ! -e "$RESOURCE_PATH" ]] ; then
    cat << EOM
error: Resource "$RESOURCE_PATH" not found. Run 'pod install' to update the copy resources script.
EOM
    exit 1
  fi
  case $RESOURCE_PATH in
    *.storyboard)
      echo "ibtool --reference-external-strings-file --errors --warnings --notices --minimum-deployment-target ${!DEPLOYMENT_TARGET_SETTING_NAME} --output-format human-readable-text --compile ${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename \"$RESOURCE_PATH\" .storyboard`.storyboardc $RESOURCE_PATH --sdk ${SDKROOT} ${TARGET_DEVICE_ARGS}" || true
      ibtool --reference-external-strings-file --errors --warnings --notices --minimum-deployment-target ${!DEPLOYMENT_TARGET_SETTING_NAME} --output-format human-readable-text --compile "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename \"$RESOURCE_PATH\" .storyboard`.storyboardc" "$RESOURCE_PATH" --sdk "${SDKROOT}" ${TARGET_DEVICE_ARGS}
      ;;
    *.xib)
      echo "ibtool --reference-external-strings-file --errors --warnings --notices --minimum-deployment-target ${!DEPLOYMENT_TARGET_SETTING_NAME} --output-format human-readable-text --compile ${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename \"$RESOURCE_PATH\" .xib`.nib $RESOURCE_PATH --sdk ${SDKROOT} ${TARGET_DEVICE_ARGS}" || true
      ibtool --reference-external-strings-file --errors --warnings --notices --minimum-deployment-target ${!DEPLOYMENT_TARGET_SETTING_NAME} --output-format human-readable-text --compile "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename \"$RESOURCE_PATH\" .xib`.nib" "$RESOURCE_PATH" --sdk "${SDKROOT}" ${TARGET_DEVICE_ARGS}
      ;;
    *.framework)
      echo "mkdir -p ${TARGET_BUILD_DIR}/${FRAMEWORKS_FOLDER_PATH}" || true
      mkdir -p "${TARGET_BUILD_DIR}/${FRAMEWORKS_FOLDER_PATH}"
      echo "rsync --delete -av "${RSYNC_PROTECT_TMP_FILES[@]}" $RESOURCE_PATH ${TARGET_BUILD_DIR}/${FRAMEWORKS_FOLDER_PATH}" || true
      rsync --delete -av "${RSYNC_PROTECT_TMP_FILES[@]}" "$RESOURCE_PATH" "${TARGET_BUILD_DIR}/${FRAMEWORKS_FOLDER_PATH}"
      ;;
    *.xcdatamodel)
      echo "xcrun momc \"$RESOURCE_PATH\" \"${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename "$RESOURCE_PATH"`.mom\"" || true
      xcrun momc "$RESOURCE_PATH" "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename "$RESOURCE_PATH" .xcdatamodel`.mom"
      ;;
    *.xcdatamodeld)
      echo "xcrun momc \"$RESOURCE_PATH\" \"${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename "$RESOURCE_PATH" .xcdatamodeld`.momd\"" || true
      xcrun momc "$RESOURCE_PATH" "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename "$RESOURCE_PATH" .xcdatamodeld`.momd"
      ;;
    *.xcmappingmodel)
      echo "xcrun mapc \"$RESOURCE_PATH\" \"${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename "$RESOURCE_PATH" .xcmappingmodel`.cdm\"" || true
      xcrun mapc "$RESOURCE_PATH" "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}/`basename "$RESOURCE_PATH" .xcmappingmodel`.cdm"
      ;;
    *.xcassets)
      ABSOLUTE_XCASSET_FILE="$RESOURCE_PATH"
      XCASSET_FILES+=("$ABSOLUTE_XCASSET_FILE")
      ;;
    *)
      echo "$RESOURCE_PATH" || true
      echo "$RESOURCE_PATH" >> "$RESOURCES_TO_COPY"
      ;;
  esac
}
if [[ "$CONFIGURATION" == "Debug" ]]; then
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/LMFileMoveBaseViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/LMFileMoveMainVC.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/LMFileMoveWnController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/circle_icon.imageset/circle_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/circle_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset/disk_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset/电脑硬盘@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_gray.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_gray.imageset/有外存储=off@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_hight.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_hight.imageset/disk_icon_hight@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_text_color.colorset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/dot_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/dot_icon.imageset/dot_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_icon.imageset/file_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_done_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_done_icon.imageset/file_move_loading_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_error_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_error_icon.imageset/file_move_error_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_pending_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_pending_icon.imageset/file_move_pending_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_done.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_done.imageset/file_move_result_circle_done.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_space.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_space.imageset/file_move_result_circle.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_done_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_done_icon.imageset/file_move_result_done.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_warning_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_warning_icon.imageset/file_move_warning_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/finder_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/finder_icon.imageset/finder_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/gray_disk_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/gray_disk_icon.imageset/gray_disk_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/image_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/image_icon.imageset/image_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_big_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_big_icon.imageset/qq_big_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_icon.imageset/qq_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/video_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/video_icon.imageset/video_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/view_bg_color.colorset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_big_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_big_icon.imageset/wecom_big_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_icon.imageset/wecom_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_big_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_big_icon.imageset/wx_big_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_icon.imageset/wx_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/文件搬家.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/文件搬家.imageset/文件搬家@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/circle_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_gray.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_hight.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_text_color.colorset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/dot_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_done_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_error_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_pending_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_done.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_space.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_done_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_warning_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/finder_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/gray_disk_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/image_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_big_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/video_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/view_bg_color.colorset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_big_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_big_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/文件搬家.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/zh-Hans.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset/刷新-normal.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset/刷新-normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset/刷新-down.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset/刷新-down@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset/刷新-hover.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset/刷新-hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset/normal@1x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset/normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/icon_pc.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/icon_pc.imageset/pc.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset/kernel_extension.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset/kernel_extension@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset/login_item.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset/login_item@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/no_app_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/no_app_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/no_app_icon@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset/running_app.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset/running_app@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_ch.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_ch.imageset/中文@2x(1).png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_en.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_en.imageset/英文@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/LmTrashWatchUninstallWindowController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallDetailViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallSelectedViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallWindowController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/get_pkg_path.sh"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/uninstallConfigs.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/icon_pc.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_ch.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_en.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/zh-Hans.lproj"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_disable.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_disable@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_hover.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_normal.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_press.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_press@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_disable.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_disable@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_hover.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_normal.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_press.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_press@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_disable.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_disable@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_hover.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_normal.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_press.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_press@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMLoadingView/loading.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMLoadingView/loading@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMUICommon.xcassets"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/LemonHardwareViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/LemonHardwareWindowController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/电池@2x-1.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/电池@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/电池@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset/电池充满@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset/电池充满@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset/电池充电中@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset/电池充电中@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/托盘态 copy 2@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/托盘态 copy 2@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/电脑硬盘@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/托盘态 copy 2@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/托盘态 copy 2@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/显卡@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset/ic_analyze.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset/ic_analyze@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/内存 515151@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/内存 515151@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/内存@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/处理器@2x-1.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/处理器@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/处理器@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset/triangleButton.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset/triangleButton@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset/triangleButtonSelected.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset/triangleButtonSelected@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/zh-Hans.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset/defaultTeminate@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset/defaultTeminate@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset/Fill 1 Copy 4@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset/Fill 1 Copy 4@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_disconnect_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_disconnect_icon.imageset/网络检测 断网@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_icon.imageset/网络检测@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/仪表.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/仪表@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/仪表@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/断网 2.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/断网@2x 2.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/断网@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/空白状态.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/空白状态@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/空白状态@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset/空白状态.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset/空白状态@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/upload_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/upload_icon.imageset/Fill 1 Copy 4.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/LMNetProcViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/LMNetProcWndController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/LMProcessPortViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/port.plist"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_disconnect_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/upload_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/zh-Hans.lproj"
fi
if [[ "$CONFIGURATION" == "Release" ]]; then
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/LMFileMoveBaseViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/LMFileMoveMainVC.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/LMFileMoveWnController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/circle_icon.imageset/circle_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/circle_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset/disk_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset/电脑硬盘@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_gray.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_gray.imageset/有外存储=off@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_hight.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_hight.imageset/disk_icon_hight@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_text_color.colorset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/dot_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/dot_icon.imageset/dot_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_icon.imageset/file_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_done_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_done_icon.imageset/file_move_loading_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_error_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_error_icon.imageset/file_move_error_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_pending_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_pending_icon.imageset/file_move_pending_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_done.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_done.imageset/file_move_result_circle_done.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_space.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_space.imageset/file_move_result_circle.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_done_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_done_icon.imageset/file_move_result_done.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_warning_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_warning_icon.imageset/file_move_warning_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/finder_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/finder_icon.imageset/finder_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/gray_disk_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/gray_disk_icon.imageset/gray_disk_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/image_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/image_icon.imageset/image_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_big_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_big_icon.imageset/qq_big_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_icon.imageset/qq_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/video_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/video_icon.imageset/video_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/view_bg_color.colorset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_big_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_big_icon.imageset/wecom_big_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_icon.imageset/wecom_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_big_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_big_icon.imageset/wx_big_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_icon.imageset/wx_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/文件搬家.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/文件搬家.imageset/文件搬家@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/circle_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_gray.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_icon_hight.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/disk_text_color.colorset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/dot_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_done_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_error_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_pending_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_done.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_circle_space.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_result_done_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/file_move_warning_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/finder_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/gray_disk_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/image_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_big_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/qq_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/video_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/view_bg_color.colorset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_big_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wecom_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_big_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/wx_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/Media.xcassets/文件搬家.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonFileMove/LemonFileMove/Assets/zh-Hans.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset/刷新-normal.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset/刷新-normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset/刷新-down.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset/刷新-down@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset/刷新-hover.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset/刷新-hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset/normal@1x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset/normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/icon_pc.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/icon_pc.imageset/pc.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset/kernel_extension.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset/kernel_extension@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset/login_item.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset/login_item@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/no_app_icon.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/no_app_icon@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset/no_app_icon@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset/running_app.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset/running_app@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_ch.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_ch.imageset/中文@2x(1).png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_en.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_en.imageset/英文@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/LmTrashWatchUninstallWindowController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallDetailViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallSelectedViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/McUninstallWindowController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/get_pkg_path.sh"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/uninstallConfigs.json"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_d.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/btn_refresh_hl.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/filter_normal.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/icon_pc.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/kernel_extension.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/login_item.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/no_app_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/running_app.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_ch.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Assets.xcassets/setstep_en.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonUninstaller/LemonUninstaller/Assets/Resources/zh-Hans.lproj"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_disable.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_disable@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_hover.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_normal.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_press.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/mix_select_press@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_disable.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_disable@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_hover.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_normal.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_press.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/selected_press@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_disable.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_disable@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_hover.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_hover@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_normal.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_normal@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_press.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMCheckboxButton/unselected_press@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMLoadingView/loading.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMLoadingView/loading@2x.png"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/QMUICommon.xcassets"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/QMUICommon/QMUICommon/Assets/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/LemonHardwareViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/LemonHardwareWindowController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/电池@2x-1.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/电池@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset/电池@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset/电池充满@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset/电池充满@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset/电池充电中@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset/电池充电中@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/托盘态 copy 2@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/托盘态 copy 2@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset/电脑硬盘@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/托盘态 copy 2@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/托盘态 copy 2@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset/显卡@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset/ic_analyze.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset/ic_analyze@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/内存 515151@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/内存 515151@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset/内存@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/处理器@2x-1.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/处理器@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset/处理器@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset/triangleButton.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset/triangleButton@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset/triangleButtonSelected.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset/triangleButtonSelected@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/battery.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_full.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/charge_in.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/disk.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/graphic.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/ic_analyze.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/memory.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/processor.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButton.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/Assets.xcassets/triangleButtonSelected.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonHardware/LemonHardware/Assets/Resources/zh-Hans.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset/defaultTeminate@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset/defaultTeminate@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset/Fill 1 Copy 4@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset/Fill 1 Copy 4@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_disconnect_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_disconnect_icon.imageset/网络检测 断网@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_icon.imageset/网络检测@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/仪表.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/仪表@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset/仪表@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/断网 2.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/断网@2x 2.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset/断网@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/空白状态.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/空白状态@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset/空白状态@3x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset/空白状态.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset/空白状态@2x.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/upload_icon.imageset/Contents.json"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/upload_icon.imageset/Fill 1 Copy 4.png"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/LMNetProcViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/LMNetProcWndController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/LMProcessPortViewController.xib"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/en.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/port.plist"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/zh-Hans.lproj/Localizable.strings"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/defaultTeminate.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/download_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_disconnect_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/net_speed_meter.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_net_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_bg.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/no_proc_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Asset.xcassets/upload_icon.imageset"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/en.lproj"
  install_resource "${PODS_ROOT}/../localPod/LemonNetSpeed/LemonNetSpeed/Assets/Resources/zh-Hans.lproj"
fi

mkdir -p "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}"
rsync -avr --copy-links --no-relative --exclude '*/.svn/*' --files-from="$RESOURCES_TO_COPY" / "${TARGET_BUILD_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}"
if [[ "${ACTION}" == "install" ]] && [[ "${SKIP_INSTALL}" == "NO" ]]; then
  mkdir -p "${INSTALL_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}"
  rsync -avr --copy-links --no-relative --exclude '*/.svn/*' --files-from="$RESOURCES_TO_COPY" / "${INSTALL_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}"
fi
rm -f "$RESOURCES_TO_COPY"

if [[ -n "${WRAPPER_EXTENSION}" ]] && [ "`xcrun --find actool`" ] && [ -n "${XCASSET_FILES:-}" ]
then
  # Find all other xcassets (this unfortunately includes those of path pods and other targets).
  OTHER_XCASSETS=$(find -L "$PWD" -iname "*.xcassets" -type d)
  while read line; do
    if [[ $line != "${PODS_ROOT}*" ]]; then
      XCASSET_FILES+=("$line")
    fi
  done <<<"$OTHER_XCASSETS"

  if [ -z ${ASSETCATALOG_COMPILER_APPICON_NAME+x} ]; then
    printf "%s\0" "${XCASSET_FILES[@]}" | xargs -0 xcrun actool --output-format human-readable-text --notices --warnings --platform "${PLATFORM_NAME}" --minimum-deployment-target "${!DEPLOYMENT_TARGET_SETTING_NAME}" ${TARGET_DEVICE_ARGS} --compress-pngs --compile "${BUILT_PRODUCTS_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}"
  else
    printf "%s\0" "${XCASSET_FILES[@]}" | xargs -0 xcrun actool --output-format human-readable-text --notices --warnings --platform "${PLATFORM_NAME}" --minimum-deployment-target "${!DEPLOYMENT_TARGET_SETTING_NAME}" ${TARGET_DEVICE_ARGS} --compress-pngs --compile "${BUILT_PRODUCTS_DIR}/${UNLOCALIZED_RESOURCES_FOLDER_PATH}" --app-icon "${ASSETCATALOG_COMPILER_APPICON_NAME}" --output-partial-info-plist "${TARGET_TEMP_DIR}/assetcatalog_generated_info_cocoapods.plist"
  fi
fi
